#encoding: utf-8
#----------------------------------------
# Mod indépendant (nouvelles constantes)
#----------------------------------------
# Ce script permet de paramétrer en un seul endroit les constantes, variables, booléens et arrays utilisés dans ce mod.

DEATH_OVERHAUL = true # Ne pas toucher, permet au moteur de jeu de détecter la présence du mod. (Très utile pour une compatibilité automatique de mes autres mods.)

# ---------- Partie à paramétrer par le maker ----------

module Yuki
	module Sw #Liste des switchs RMXP à dédier pour ce mod. Attribuez ici les numéros que vous souhaitez.
		EnemyDeath = 131 # Mortalité des Pokémons adverses. Valeur à "true" si on peut tuer l'ennemi, "false" si on ne peut pas.
		AreaClause = 164 # Permet d'activer de manière indépendante la Règle d'or n°1 du Nuzlocke : un seul Pokémon capturable par zone.
		DupesClause = 165 # Permet d'activer la clause doublon. (Si le premier Pokémon rencontré dans une zone est déjà dans le Pokédex, la zone n'est pas bloquée malgré l'AreaClause.)
		ShinyClause = 166 # Permet d'activer la clause chromatique. (Si un shiny est rencontré, il peut être capturé sans restriction malgré l'AreaClause.)
		LegendaryClause = 167 # Permet d'autoriser la capture des Pokémons légendaires en Nuzlocke, même s'ils ne sont pas le Pokémon de zone au moment de la rencontre. Dépendant de l'array LEG.
		BlackOutClause = 168 # À activer pour que la mort de tous les Pokémons de l'équipe du joueur aboutisse directement sur un Game Over, même s'il reste des survivants au PC.
		OldSchoolPoison = 169 # Permet d'activer le poison à l'ancienne, c'est-à-dire d'avoir le Pokémon KO dès qu'il lui reste 0 PV in-map (ou mort si le Nuzlocke est activé).
	end
end


module Yuki
	module Var #Liste des variables RMXP à dédier pour ce mod. Attribuez ici les numéros que vous souhaitez. Faire pointer sur une variable qui restera à 0 si vous n'utilisez pas la feature.
		TimeElapsed = 112 # Mettre ici la variable utilisée pour mesurer le nombre de jours écoulés. Utilisée par la feature de réincarnation. À noter que si vous mettez REINCARNATION = true, mais que TimeElapsed pointe sur une variable qui reste figée ; la résurrection par les Cendres Sacrées sera possible sans limite de temps, mais les mécanismes de réincarnation n'auront pas lieu. 
		Difficulty = 122 # Utilisée dans le script "16031 Difficulty level.rb". -1 en mode facile, 0 en mode normal, 1 en mode difficile.
		Mortality = 124 # Mortalité de vos Pokémons. Valeur à 0 si désactivé, 1 si réaliste (mort si perte du double des PV max), 2 si mort immédiate dès le KO. Mettre sur 2 permet d'appliquer la Règle d'or n°2 du Nuzlocke : à combiner avec l'activation du switch AreaClause pour avoir un Nuzlocke complet, mais je permets d'avoir ces deux règles indépendantes si jamais un maker veut instaurer la possibilité d'avoir des Nuzlockes aux règles spéciales (ChainLocke, EggLocke, MonoLocke...)
		NoItemOnBattle = 174 # Permet de restreindre l'utilisation des objets en combat. 0 = Pas de restriction (vanilla), 1 et + : divers niveaux de restrictions avec un message différents (cf PDF d'installation).
	end
end

#ID des Pokémons considérés comme légendaires (utilisé pour la LegendaryClause, mais aussi pour leur mécanisme spécial de réincarnation si REINCARNATION = true.)
LEG = [144,145,146,150,151,243,244,245,249,250,251,377,378,379,380,381,382,383,384,385,386,480,481,482,483,484,485,486,487,488,490,491,492,493,494,638,639,640,641,642,643,644,645,646,647,648,649,716,717,718,719,720,721,772,773,785,786,787,788,789,790,791,792,800,801,802,807,808,809]

# Mettre "true" pour activer un stade temporaire de mort réversible où la resurrection via les Cendres Sacrées est possible + si la variable TimeElapsed n'est pas figée, la feature de réincarnation lorsque le SOUL_COOLDOWN expirera. Mettre "false" pour avoir une vraie permadeath.
REINCARNATION = true 

#Nombre entier : Jours entre mort et mort définitif (paramétrable) (Non utilisé si REINCARNATION = false)
SOUL_COOLDOWN = 3 

# Numéro de l'évent commun utilisé pour l'affichage des textes de réincarnation. (Non utilisé si REINCARNATION = false)
COMMON_EVENT_DEATH_TEXTS = 73

#Les numéros de maps où les oeufs peuvent apparaître.
#Arrays de forme [[MAP_A_ID,MAP_B_ID,...],[EVENT_A_ID,EVENT_B_ID,...]]
EGG_POS =
{151 => [[49,49],[88,89]],					#Mew (HALL DE TEST)
243 => [[49,49],[65,66]],					#Raikou
244 => [[49,49],[65,66]],					#Entei
245 => [[49],[49]],							#Suicune
251 => [[49],[49]],							#Celebi
487 => [[49,49,49,49,49],[83,84,86,87,88]]	#Giratina (HALL DE TEST)
}

#Correspondance entre l'ID du Poké mort et l'ID du spectre en lequel il se réincarne.
#POKE_ID => GHOST_ID
GHOST_ID = {104 => 105,105 => 105,115 => 105,222 => 222} # Exemple ici : La lignée Osselait (forme n°0) devient Ossatueur, et Corayon (forme n°0) devient Corayon. Leur passage en forme n°1 (spectre) est gérée un peu plus bas.

#Correspondance par type
#Numéro de type (trouvable sur RubyHost) => ID du spectre 
GHOST_TYPE = {5=>708, 3=>592, 16=>679, 2=>607, 8=>92, 9=>622, 17=>302, 12=>292}

GUARANTEE_REBIRTH = [105,115,222] #Liste des Pokés qui ont une probabilité de 100% de réincarnation à la fin du SOUL_COOLDOWN.

# ---------- Fin de la partie à paramétrer ----------

#Ensemble de fonctions gérant la gestion de la mort IG

def death_count
	return if REINCARNATION == false # Méthode inutile si la feature de réincarnation est désactivée. Ça permet aussi de se passer de l'évent commun.
	$names = [] #Nom des Pokémon qui vont mourir
	$names_s = [] #Noms des spectres réincarnés
	$id = [] #L'id du message affiché.
	
	$pokemon_party.actors.each do |pokemon|
		next unless pokemon.death_c
		if pokemon.death_c <= $game_variables[Yuki::Var::TimeElapsed]
			pokemon.death_c = nil #On sort le Pokémon traité de la liste d'attente
			if LEG.any?(pokemon.id) #Cas des Pokémon légendaires
				if resurrect?(pokemon) #Le légendaire reste avec le joueur sous forme d'oeuf
					$names.push(pokemon.given_name)
					id = pokemon.id
					$actors.delete(pokemon)
					add_egg(id)
					$id.push(2)
				else #Le légendaire renaît autre part sous forme d'un oeuf
					$names.push(pokemon.given_name)
					pokemon.status = GameData::States::Mort_d
					egg_g(pokemon)
					$id.push(3)
				end
			elsif resurrect?(pokemon) #Le Pokémon se réincarne
				$names.push(pokemon.given_name)
				pokemon.form = 0
				pokemon.id = resurrect_id(pokemon)
				# ---------- Gestion des formes spéciales (je m'incruste ici) ----------
				if pokemon.id == 105 or pokemon.id == 222
					pokemon.form = 1 # Forme spectrale (Ossatueur d'Alola / Corayon de Galar)
				end
				$names_s.push(pokemon.given_name)
				pokemon.status = 0
				pokemon.hp = pokemon.max_hp
				assign_move(pokemon)
				$id.push(1)
			else #Le Pokémon est perdu
				$names.push(pokemon.given_name)
				pokemon.status = GameData::States::Mort_d #On affiche le texte
				$id.push(0)
			end
		end
	end
	$game_temp.common_event_id = COMMON_EVENT_DEATH_TEXTS #On affiche le texte des morts formaté correctement grâce à cet évent commun.
end


#Permet de savoir si le Pokémon se réincarne
def resurrect?(pokemon)
	# return true #(Permet de court-circuiter le test d'amitié pour la réincarnation et donc de toujours avoir la réincarnation si cette ligne est déneutralisée.
	return true if GUARANTEE_REBIRTH.any?(pokemon.id)
	l=pokemon.loyalty
# Tant que le mod sur l'amitié n'est pas terminé, on reste sur l'ancienne échelle à 255.
	if l == 0 or l >= 180 #40% - 720/1023
		return true if rand(100) <= 39
	elsif l >= 150 #30% - 600/1023
		return true if rand(100) <= 29
	elsif l < 25 #20% - 100/1023
		return true if rand(100) <= 19
	elsif l >= 125 #15% - 500/1023
		return true if rand(100) <= 14
	elsif l < 50 #5% - 200/1023
		return true if rand(100) <= 4
	end
		return false
end

#Permet de récupérer le ID du bon spectre
def resurrect_id(pokemon)
	return GHOST_ID[pokemon.id] if GHOST_ID.key?(pokemon.id) #Dans le cas où la correspondance est renseigné
	return GHOST_TYPE[pokemon.type1] if GHOST_TYPE.key?(pokemon.type1)
	return GHOST_TYPE[pokemon.type2] if GHOST_TYPE.key?(pokemon.type2)
	r = rand(3)
	return 200 if r==0
	return 353 if r==1
	return 355
end

#Permet de donner les bonnes attaques au Pokémon réincarné
def assign_move(pokemon)
	#On n'oublie pas le capacité d'index 0 par précaution
	pokemon.forget_skill_index(3)
	pokemon.forget_skill_index(2)
	pokemon.forget_skill_index(1)
	movepool = GameData::Pokemon.all[pokemon.id][pokemon.form].move_set
	i = 0
	l = movepool.length
	if pokemon.level >= movepool[l-2]
		pokemon.replace_skill_index(0, movepool[l-1]) #On part du principe qu'il y a au moins une attaque dans le movepool
		pokemon.replace_skill_index(1, movepool[l-3]) if l >= 3
		pokemon.replace_skill_index(2, movepool[l-5]) if l >= 5
		pokemon.replace_skill_index(3, movepool[l-7]) if l >= 7
	else
		until movepool[i] >= pokemon.level
			i += 2
		end
		if pokemon.level == movepool[i]
			pokemon.replace_skill_index(0, movepool[i+1])
			pokemon.replace_skill_index(1, movepool[i-1]) if i >= 2
			pokemon.replace_skill_index(2, movepool[i-3]) if i >= 4
			pokemon.replace_skill_index(3, movepool[i-5]) if i >= 6
		else
			pokemon.replace_skill_index(0, movepool[i-1])
			pokemon.replace_skill_index(1, movepool[i-3]) if i >= 3
			pokemon.replace_skill_index(2, movepool[i-5]) if i >= 5
			pokemon.replace_skill_index(3, movepool[i-7]) if i >= 7
		end
	end
end
	
#Permet de faire apparaître l'oeuf en cas de la mort d'un légendaire
def egg_g(pokemon)
	maps = EGG_POS[pokemon.id][0]
	events = EGG_POS[pokemon.id][1]
	refresh = false
	i=0
	loop do
		i=rand(maps.length)
		j=events[i]
		#Pour éviter de bloquer le joueur
		next if maps[i] == $game_map.map_id and $game_map.events[j].x==$game_player.x and $game_map.events[j].y==$game_player.y
		refresh = true if maps[i] == $game_map.map_id
		break
	end
	j = events[i]
	k = maps[i]
	$game_self_switches[[k, j, 'A']] = true
	$game_map.events[j].refresh if refresh
end
